@@ -13,12 +13,20 @@ rescue Exception
   # Mongrel not available
 end
 
+begin
+  require_library_or_gem 'thin'
+rescue Exception
+  # Thin not available
+end
+
 server = case ARGV.first
-  when "lighttpd", "mongrel", "new_mongrel", "webrick"
+  when "lighttpd", "mongrel", "new_mongrel", "webrick", "thin"
     ARGV.shift
   else
     if defined?(Mongrel)
       "mongrel"
+    elsif defined?(Thin)
+      "thin"
     elsif RUBY_PLATFORM !~ /(:?mswin|mingw)/ && !silence_stderr { `lighttpd -version` }.blank? && defined?(FCGI)
       "lighttpd"
     else
@@ -33,6 +41,8 @@ case server
     puts "=> Booting lighttpd (use 'script/server webrick' to force WEBrick)"
   when "mongrel", "new_mongrel"
     puts "=> Booting Mongrel (use 'script/server webrick' to force WEBrick)"
+  when "thin"
+    puts "=> Booting Thin (use 'script/server webrick' to force WEBrick)"
 end
 
 %w(cache pids sessions sockets).each { |dir_to_make| FileUtils.mkdir_p(File.join(RAILS_ROOT, 'tmp', dir_to_make)) }
