@@ -31,14 +31,8 @@ class SecretKeyGenerationTest < Test::Unit::TestCase
   end
 
   def test_secret_key_generation
-    assert @generator.generate_secret.length >= SECRET_KEY_MIN_LENGTH
-  end
-
-  Rails::SecretKeyGenerator::GENERATORS.each do |generator|
-    if Rails::SecretKeyGenerator.send("supports_#{generator}?")
-      define_method("test_secret_key_generation_with_#{generator}") do
-        assert @generator.send("generate_secret_with_#{generator}").length >= SECRET_KEY_MIN_LENGTH
-      end
+    assert_deprecated /ActiveSupport::SecureRandom\.hex\(64\)/ do
+      assert @generator.generate_secret.length >= SECRET_KEY_MIN_LENGTH
     end
   end
 end
